@@ -30,6 +30,14 @@ import urllib2
 from HTMLParser import HTMLParser
 import re
 
+try:
+    import osgeo.gdal as gdal
+except ImportError:
+    try:
+        import gdal
+    except ImportError:
+        raise('Python GDAL library did not find, please install it')
+
 
 def urljoin(*args):
     """
@@ -125,7 +133,8 @@ class downModis:
                 enddate=None,
                 delta=10,
                 jpg=False,
-                debug=False
+                debug=False,
+                timeout=30
               ):
         """Initialization function :
 
@@ -163,6 +172,7 @@ class downModis:
 
             debug = set True if you want to obtain debug information
 
+            timeout = Timeout value for http server
         """
 
         # url modis
@@ -221,6 +231,16 @@ class downModis:
         logging.basicConfig(filename=log_filename, level=logging.DEBUG, \
         format=log_format)
         self.nconnection = 0
+        self.timeout = timeout
+        self.fileInPath = []
+        # add all files in the directory where we will save new modis data
+        for f in os.listdir(self.writeFilePath):
+            if os.path.isfile(os.path.join(self.writeFilePath, f)):
+                self.fileInPath.append(f)
+        gdal.UseExceptions()
+        gdalDriver = gdal.GetDriverByName('HDF4')
+        if not gdalDriver:
+            raise IOError("GDAL has no support for HDF4, please update GDAL")
 
     def removeEmptyFiles(self):
         """Check if some file has size ugual 0"""
@@ -249,11 +269,18 @@ class downModis:
         """
         self.nconnection += 1
         try:
-            http = urllib2.urlopen(urljoin(self.url, self.path))
+            http = urllib2.urlopen(urljoin(self.url, self.path),
+                                   timeout=self.timeout)
+        except:
+            logging.error('Error in HTTP connection')
+            if self.nconnection <= ncon:
+                self._connectHTTP()
+        try:
             self.dirData = modisHtmlParser(http).get_dates()
             self.dirData.reverse()
         except (EOFError, urllib2.URLError), e:
             logging.error('Error in connection: %s' % e)
+            self.closeHTTP(http)
             if self.nconnection <= ncon:
                 self._connectHTTP()
 
@@ -292,6 +319,16 @@ class downModis:
         if self.debug == True:
             logging.debug("Close connection %s" % self.url)
 
+    def closeHTTP(self, http):
+        """ Close http connection
+
+            http = the http concention object created by urlib2.open         
+        """
+        http.close()
+        self.closeFilelist()
+        if self.debug == True:
+            logging.debug("Close connection %s" % self.url)
+
     def closeFilelist(self):
         """ Function to close the file where write the downloaded files """
         self.filelist.close()
@@ -323,9 +360,9 @@ class downModis:
             # set enday variable to data
         if self.enday != None:
             self.enday = str2date(self.enday)
-        if self.today < self.enday:
-            raise IOError("The first day should be newer then end date")
         if self.today and self.enday:
+            if self.today < self.enday:
+                raise IOError("The first day should be newer then end date")
             D = self.today - self.enday
             self.delta = D.days
 
@@ -388,7 +425,8 @@ class downModis:
             url = urljoin(self.url, self.path, day)
             if self.debug == True:
                 logging.debug("The url is: %s" % url)
-            http = modisHtmlParser(urllib2.urlopen(url))
+            conn = urllib2.urlopen(url)
+            http = modisHtmlParser(conn)
             # download also jpeg
             if self.jpeg:
                 # finallist is ugual to all file with jpeg file
@@ -405,6 +443,7 @@ class downModis:
                 logging.debug("The number of file to download is: %i" % len(finalList))
             return finalList
         except (socket.error), e:
+            self.closeHTTP(conn)
             logging.error("Error %s when try to receive list of files" % e)
             self._getFilesListHTTP(day)
 
@@ -460,25 +499,35 @@ class downModis:
             move = it is useful to know if a function is called from download
                    or move function
         """
-        fileInPath = []
-        # add all files in the directory where we will save new modis data
-        for f in os.listdir(self.writeFilePath):
-            if os.path.isfile(os.path.join(self.writeFilePath, f)):
-                fileInPath.append(f)
         # different return if this method is used from downloadsAllDay() or
         # moveFile()
         if move == 0:
-            listOfDifferent = list(set(listNewFile) - set(fileInPath))
+            listOfDifferent = list(set(listNewFile) - set(self.fileInPath))
         elif move == 1:
-            listOfDifferent = list(set(fileInPath) - set(listNewFile))
+            listOfDifferent = list(set(self.fileInPath) - set(listNewFile))
+        if self.debug == True:
+                logging.debug("The number of file to download after check of" \
+                              "existing file is: %i" % len(listOfDifferent))
         return listOfDifferent
 
+    def checkFile(self, filHdf):
+        """Check using GDAL to be sure that download was fine
+
+        filHdf = name of the HDF file to check
+        """
+        try:
+            gdal.Open(filHdf)
+            return 0
+        except (RuntimeError), e:
+            logging.error(e)
+            return 1
+
     def downloadFile(self, filDown, filHdf, day):
         """Download the single file
 
         filDown = name of the file to download
 
-        filSave = name of the file to write
+        filHdf = name of the file to write
 
         day = the day in format YYYY.MM.DD
         """
@@ -502,19 +551,31 @@ class downModis:
             orig_size = http.headers['content-length']
             filSave.write(http.read())
             filSave.close()
-            self.filelist.write("%s\n" % filDown)
-            if self.debug == True:
-                logging.debug("File %s downloaded" % filDown)
         #if it have an error it try to download again the file
         except (EOFError), e:
             logging.error("Cannot download %s, the error was '%s'. Retry.." % (
                           filDown, e))
             filSave.close()
             os.remove(filSave.name)
+            self.closeHTTP(http)
             self._downloadFileHTTP(filDown, filHdf, day)
         transf_size = os.path.getsize(filSave.name)
         if int(orig_size) == int(transf_size):
-            return 0
+            if filHdf.find('.xml') == -1:
+                if self.checkFile(filHdf):
+                    os.remove(filSave.name)
+                    self.closeHTTP(http)
+                    self._downloadFileHTTP(filDown, filHdf, day)
+                else:
+                    self.filelist.write("%s\n" % filDown)
+                    if self.debug == True:
+                        logging.debug("File %s downloaded correctly" % filDown)
+                    return 0
+            else:
+                self.filelist.write("%s\n" % filDown)
+                if self.debug == True:
+                    logging.debug("File %s downloaded correctly" % filDown)
+                return 0
         else:
             logging.warning("Different size for file %s - original data: %s,"\
                             " downloaded: %s" % (filDown, orig_size,
@@ -585,6 +646,8 @@ class downModis:
             elif numFiles > 1:
                 logging.error("There are to much files for %s" % i)
             if numFiles == 0 or (numFiles == 1 and fileDown != oldFile[0]):
+                if self.debug == True:
+                    logging.debug("Start to download file %s" % i)
                 self.downloadFile(i, file_hdf, day)
 
     def downloadsAllDay(self, clean=False, allDays=False):
@@ -606,7 +669,7 @@ class downModis:
         if self.urltype == 'http':
             self._downloadsAllDayHTTP(days)
         elif self.urltype == 'ftp':
-            self._downloadFileFTP(days)
+            self._downloadAllDayFTP(days)
 
     def _downloadsAllDayHTTP(self, days):
         """ Downloads all the tiles considered from HTTP server"""
